/**
 * DevExtreme (ui/text_box/action_button_collection/index.js)
 * Version: 19.1.0 (build 19071)
 * Build date: Tue Mar 12 2019
 *
 * Copyright (c) 2012 - 2019 Developer Express Inc. ALL RIGHTS RESERVED
 * Read about DevExtreme licensing here: https://js.devexpress.com/Licensing/
 */
"use strict";
Object.defineProperty(exports, "__esModule", {
    value: true
});
var _createClass = function() {
    function defineProperties(target, props) {
        for (var i = 0; i < props.length; i++) {
            var descriptor = props[i];
            descriptor.enumerable = descriptor.enumerable || false;
            descriptor.configurable = true;
            if ("value" in descriptor) {
                descriptor.writable = true
            }
            Object.defineProperty(target, descriptor.key, descriptor)
        }
    }
    return function(Constructor, protoProps, staticProps) {
        if (protoProps) {
            defineProperties(Constructor.prototype, protoProps)
        }
        if (staticProps) {
            defineProperties(Constructor, staticProps)
        }
        return Constructor
    }
}();
var _renderer = require("../../../core/renderer");
var _renderer2 = _interopRequireDefault(_renderer);
var _array = require("../../../core/utils/array");

function _interopRequireDefault(obj) {
    return obj && obj.__esModule ? obj : {
        "default": obj
    }
}

function _classCallCheck(instance, Constructor) {
    if (!(instance instanceof Constructor)) {
        throw new TypeError("Cannot call a class as a function")
    }
}
var TEXTEDITOR_BUTTONS_CONTAINER_CLASS = "dx-texteditor-buttons-container";
var ActionButtonCollection = function() {
    function ActionButtonCollection(editor, defaultButtonsInfo) {
        _classCallCheck(this, ActionButtonCollection);
        this.buttons = [];
        this.defaultButtonsInfo = defaultButtonsInfo;
        this.editor = editor
    }
    _createClass(ActionButtonCollection, [{
        key: "_createButton",
        value: function(buttonName) {
            var defaultButtonInfo = (0, _array.find)(this.defaultButtonsInfo, function(_ref) {
                var name = _ref.name;
                return name === buttonName
            });
            if (!defaultButtonInfo) {
                throw "Can't create custom action button"
            }
            var Ctor = defaultButtonInfo.Ctor;
            var button = new Ctor(this.editor, {});
            this.buttons.push(button);
            return button
        }
    }, {
        key: "_renderButtons",
        value: function(buttons, $container, location) {
            var _this = this;
            var $buttonsContainer = $buttonsContainer = (0, _renderer2.default)("<div>").addClass(TEXTEDITOR_BUTTONS_CONTAINER_CLASS).appendTo($container);
            if (!buttons) {
                buttons = this.defaultButtonsInfo.map(function(_ref2) {
                    var name = _ref2.name;
                    return name
                })
            }
            buttons.forEach(function(buttonName) {
                var button = (0, _array.find)(_this.buttons, function(_ref3) {
                    var name = _ref3.name;
                    return name === buttonName
                });
                button = button || _this._createButton(buttonName);
                if (button.location === location) {
                    button.render($buttonsContainer)
                }
            });
            if (!$buttonsContainer.children().length) {
                $buttonsContainer.remove();
                return null
            }
            return $buttonsContainer
        }
    }, {
        key: "clean",
        value: function() {
            this.buttons.forEach(function(button) {
                return button.dispose()
            });
            this.buttons = []
        }
    }, {
        key: "getButton",
        value: function(buttonName) {
            var button = (0, _array.find)(this.buttons, function(_ref4) {
                var name = _ref4.name;
                return name === buttonName
            });
            if (!button) {
                throw "Cannot find button with this name"
            }
            return button.instance
        }
    }, {
        key: "renderAfterButtons",
        value: function(buttons, $container) {
            return this._renderButtons(buttons, $container, "after")
        }
    }, {
        key: "renderBeforeButtons",
        value: function(buttons, $container) {
            return this._renderButtons(buttons, $container, "before")
        }
    }, {
        key: "updateButtons",
        value: function(names) {
            this.buttons.forEach(function(button) {
                if (!names || names.indexOf(button.name) !== -1) {
                    button.update()
                }
            })
        }
    }]);
    return ActionButtonCollection
}();
exports.default = ActionButtonCollection;
